==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ronportal/controller/downloadcontroller.php <==
<?php
/**
 * stanislav.elbl, 11.03.2021
 * Download dat (kalendare ical atd.)
 */

namespace RonPortal\Controller;

use Ron\CalendarItem;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HttpHeaders;
use Ron\MVC\DataOutputObject;
use Ron\MVC\IcalOutputObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dochazka\AbsenceCalendarItem;
use RonPortal\NastaveniUzivatele;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\SluzebniCesty\SluzebniCestaCalItem;

class DownloadController extends JsonController
{
    /** @var  Database */
    protected $database;

    /**
     * DownloadController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        $this ->enableAccessWithoutLogin = true;
    }


    /**
     * @param $loginName
     * @param $jenSchvalene
     * @return AbsenceCalendarItem[]
     * @throws DBException
     */
    function absenceIcalEvents($loginName, $jenSchvalene)
    {
        $events = AbsenceCalendarItem::loadUserEvents($this ->database, $loginName, $jenSchvalene);
        return $events;
    }


    /**
     * @param $loginName
     * @return bool|SluzebniCestaCalItem|SluzebniCestaCalItem[]
     * @throws DBException
     */
    function sluzebniCestyIcalEvetns($loginName)
    {
        TextManager::getInstance() ->loadStrings('sluzebnicesty.ini');
        $items = SluzebniCestaCalItem::loadSluzebniCesty($this ->database, null, $loginName);
        return $items;
    }


    /**
     * @param $login
     * @return array
     * @throws DBException
     */
    function smenyIcalEvents($login)
    {
        $events = [];

        $params = $this ->database ->createParameters();

        $sql =
            'SELECT PlanovaniKapacit.Datum, PlanovaniKapacit.Smena, Smena.Popis, Smena.Variabilni, ' . // 0..3
            '       PlanovaniKapacit.Smena_Pocatek, PlanovaniKapacit.Smena_Konec, PlanovaniKapacit.Smena_Delka,' . // 4..6
            '       Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 7..9
            '       PracovniPomer.Id ' . // 10
            '  FROM PlanovaniKapacit, PracovniPomer, Smena, Osoba ' .
            ' WHERE PracovniPomer.RC = PlanovaniKapacit.RC ' .
            '   AND PracovniPomer.OsobniCislo = PlanovaniKapacit.OsobniCislo ' .
            '   AND PracovniPomer.DatumNastupu = PlanovaniKapacit.DatumNastupu ' .
            '   AND Smena.Kod = PlanovaniKapacit.Smena ' .
            '   AND Osoba.RC = PracovniPomer.RC ' .
            '   AND Osoba.IntrawebLogin = ' . $params->addParameter($login) .
            '   AND PlanovaniKapacit.Datum >= ' . $params ->addParameter($this ->database ->dateToParam(time())) .
            ' ORDER BY PlanovaniKapacit.Datum';

        $cursor = $this ->database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $cal = new CalendarItem();

            $date = $this ->database ->stringToTimestamp($row[0]);
            if ($row[3]) {
                // variabilni
                $casOd = DateUtils::timeOf($this ->database ->stringToTimestamp($row[4]));
                $casDo = DateUtils::timeOf($this ->database ->stringToTimestamp($row[5]));
            } else {
                // pevna
                $casOd = DateUtils::timeOf($this ->database ->stringToTimestamp($row[7]));
                $casDo = DateUtils::timeOf($this ->database ->stringToTimestamp($row[8]));
            }

            // OSETRIT KONEC SMENY V DALSIM DNI!

            $cal->dateTimeFrom = $date + $casOd;
            $cal->dateTimeTo = $date + $casDo;

            $cal->uid = "ronportal.pk.{$row[10]}.$date";

            $cal->summary = StringUtils::fromUTF8('Směna ') . $row[1];

            $events[] = $cal;
        }

        $cursor ->close();

        return $events;
    }


    function renderError($text, $httpStatus = 'HTTP/1.0 404 Not Found')
    {
        header($httpStatus);
        header('Content-Type: text/plain; charset=windows-1250');
        echo $text;
        exit;
    }


    /**
     * @return IcalOutputObject
     * @throws \Exception
     * @throws DBException
     */
    function downloadDataOsoby()
    {
        $usersettings = new NastaveniUzivatele();
        if (!$usersettings->load($this ->database, null, $_GET['login'])) {
            // nepovedlo se nacist nastaveni - pravdepodobne neexistuje osoba s takovym loginem
            $this->renderError('Spatne prihlasovaci jmeno / bad login name!');
        }

        if (!$usersettings->getItem(NastaveniUzivatele::ICAL_KEY)) {
            // uzivatel nema vygenerovany platny klic
            $this ->renderError('Uzivatel nema vygenerovany klic!');
        }

        // $_GET['data'] - co se ma stahnout
        $filename = isset($_GET['filename']) ? $_GET['filename'] : 'calendar.ics'; // nepovinne jmeno souboru (pro testovani mozno zadat &filename='kalendar.txt')

        if ($_GET['key'] != $usersettings->getItem(NastaveniUzivatele::ICAL_KEY)) {
            // uzivatel nema vygenerovany platny klic
            $this ->renderError('Spatne zadany klic!', 'HTTP/1.0 401 Unauthorized');
        }

        require_once('ron/calendar.php');

        $items = [];

        switch ($_GET['data']) {
            case 'absence' :
                $items = $this ->absenceIcalEvents($_GET['login'], true); // jen schvalene
                break;

            case 'absence1' :
                $items = $this ->absenceIcalEvents($_GET['login'], false); // jen schvalene = false
                break;

            case 'smeny' :
                $items = $this ->smenyIcalEvents($_GET['login']);
                break;

            case 'sluzebnicesty' :
                $items = $this ->sluzebniCestyIcalEvetns($_GET['login']);
                break;

            case 'all' :
                // absence
                $itemsAbsence = null;
                if ($usersettings ->getItem(NastaveniUzivatele::ICAL_VSECHNY_ABSENCE)) $itemsAbsence = $this ->absenceIcalEvents($_GET['login'], false);
                else if ($usersettings->getItem(NastaveniUzivatele::ICAL_SCHVALENE_ABSENCE)) $itemsAbsence = $this ->absenceIcalEvents($_GET['login'], true);
                // smeny
                $itemsSmeny = $usersettings->getItem(NastaveniUzivatele::ICAL_SMENY)
                    ? $this ->smenyIcalEvents($_GET['login'])
                    : null;
                // sluzebni cesty
                $itemsSC = $usersettings->getItem(NastaveniUzivatele::ICAL_SLUZEBNI_CESTY)
                    ? $this ->sluzebniCestyIcalEvetns($_GET['login'])
                    : null;

                if ($itemsAbsence) foreach ($itemsAbsence as $item) $items[] = $item;
                if ($itemsSmeny) foreach ($itemsSmeny as $item) $items[] = $item;
                if ($itemsSC) foreach ($itemsSC as $item) $items[] = $item;
                break;

            default :
                throw new \Exception('Neznama akce!');
        }

        return new IcalOutputObject($items, $filename);
    }


    /**
     * @return IcalOutputObject
     * @throws DBException
     */
    function downloadAbsenceHromadne()
    {
        // seznam vsech absenci
        if (!RonPortalSettings::getIntValue('absence_hromadny_ical')) {
            $this ->renderError('Tato funkce neni povolena!');
        }

        if ($_GET['key'] != RonPortalSettings::getValue('absence_ical_key')) {
            $this ->renderError('Spatne zadany klic!', 'HTTP/1.0 401 Unauthorized');
        }

        $events = AbsenceCalendarItem::loadAllEvents($this ->database, true, (int)$_GET['typrozdeleni'], $_GET['rozdeleni']);

        $filename = isset($_GET['filename']) ? $_GET['filename'] : 'calendar.ics'; // nepovinne jmeno souboru (pro testovani mozno zadat &filename='kalendar.txt')
        return new IcalOutputObject($events, $filename);
    }


    public static function changeColor($htmlColor, $step = 10)
    {
        if (!$htmlColor  ||  $htmlColor[0] != '#') {
            return $htmlColor;
        }

        if (strlen($htmlColor) == 4) {
            $r = $htmlColor[1] . $htmlColor[1];
            $g = $htmlColor[2] . $htmlColor[2];
            $b = $htmlColor[3] . $htmlColor[3];
        } else if (strlen($htmlColor) == 7) {
            $r = $htmlColor[1] . $htmlColor[2];
            $g = $htmlColor[3] . $htmlColor[4];
            $b = $htmlColor[5] . $htmlColor[6];
        } else {
            return $htmlColor;
        }

        $r = max(0, min(255, hexdec($r) + $step));
        $g = max(0, min(255, hexdec($g) + $step));
        $b = max(0, min(255, hexdec($b) + $step));

        return
            '#' .
            str_pad(dechex($r), 2, '0', STR_PAD_LEFT) .
            str_pad(dechex($g), 2, '0', STR_PAD_LEFT) .
            str_pad(dechex($b), 2, '0', STR_PAD_LEFT);
    }


    function getBasicUserCss()
    {
        if (!RonPortalSettings::getIntValue('css_pouzit_barvy')) {
            return '';
        }

        $result = '';

        // hlavicka
        if ($color = RonPortalSettings::getValue('css_header_background')) {
            $result .=
                ".header { background: $color; }\n" .
                ".header-background { background: $color !important; }\n" .
                ".floatingmenu { border-color: $color; }\n";
        }
        if ($color = RonPortalSettings::getValue('css_header_color')) {
            $result .=
                ".header { color: $color; }\n" .
                ".header-color { color: $color !important; }\n";
        }
        if ($height = RonPortalSettings::getValue('css_header_height')) {
            $result .=
                ".header { height: $height; }\n" .
                ".floatingmenu.expanded .topmenu { padding-top: $height; }\n";
        }

        // nadpisy
        if ($color = RonPortalSettings::getValue('css_text_header_color')) {
            $result .=
                "h1, h2, .appcontent-headerdiv { color: $color; }\n";
        }

        // tlacitka
        if ($color = RonPortalSettings::getValue('css_button_background')) {
            $result .=
                ".submitbutton, .button { background: $color; }\n" .
                ".button-color { background: $color !important; }\n" .
                ".submitbutton, .button { border-color: " . self::changeColor($color, -20) . "; }\n" .
                ".button:hover, .submitbutton:hover { background: " . self::changeColor($color) . "; }\n";
        }
        if ($color = RonPortalSettings::getValue('css_button_color')) {
            $result .=
                ".submitbutton, .button { color: $color; }\n" .
                ".button-color { color: $color !important; }\n";
        }

        // zahlavi tabulek
        if ($color = RonPortalSettings::getValue('css_table_header_background')) {
            $result .=
                ".browser th, .floatingheader, .floatingheadercol { background: $color; }\n" .
                ".background-table-header { background: $color !important; }\n";
        }
        if ($color = RonPortalSettings::getValue('css_table_header_color')) {
            $result .=
                ".browser th, .browser th a, .floatingheader, .floatingheader a, .floatingheadercol, .floatingheadercol a { color: $color; }\n" .
                ".color-table-header { color: $color !important; }\n";
        }


        return $result;
    }

    /**
     * @return DataOutputObject
     * @throws \Exception
     */
    function downloadUserCss()
    {
        $text = $this->getBasicUserCss();

        $cursor = $this->database->select("SELECT Val FROM SYS_BVAR_GLOBAL WHERE Name = 'RonPortal_CSS'");
        $row = $cursor->fetch();
        $text .= $row ? $cursor ->stringToBinary($row[0]) : '';
        $cursor ->close();

        //HttpResponse::setHeader(HttpHeaders::LAST_MODIFIED, date('r', RonPortalSession::sessionTimestamp()));
        //HttpResponse::setHeader(HttpHeaders::EXPIRES, date('r', DateUtils::incDay(RonPortalSession::sessionTimestamp(), 1)));
        $result = new DataOutputObject($text, 'user_styles.css');
        $result ->headers[HttpHeaders::CACHE_CONTROL] = 'private, max-age=3600';
        //$result ->headers[HttpHeaders::CACHE_CONTROL] = 'private';
        $result ->headers[HttpHeaders::PRAGMA] = 'private';
        $result ->headers[HttpHeaders::EXPIRES] = date('D, d M Y H:i:s', DateUtils::incDay(RonPortalSession::sessionTimestamp(), 1)) . ' GMT';
        return $result;
    }

    /**
     * @param JsonOutputObject $output
     * @throws DBException
     */
    function downloadObce($output)
    {
        $obce = [];

        $sql =
            'SELECT Obec.Kod, Obec.Popis, Obec.Okres, Okres.Popis' .
            ' FROM Obec JOIN Okres ON Obec.Okres = Okres.Kod' .
            ' ORDER BY Obec.Popis';

        $cursor = $this ->database->getSqlResult($sql);
        while ($row = $cursor->fetch()) {
            $obce[] = [
                'kod' => StringUtils::jsonText($row[0]),
                'popis' => StringUtils::jsonText($row[1]),
                'okres' => StringUtils::jsonText($row[2]),
                'okresPopis' => StringUtils::jsonText($row[3])
            ];
        }
        $cursor ->close();

        $output ->addData('items', $obce);
        $output->compress = 'gzip';
    }

    /**
     * @param JsonOutputObject $output
     * @throws DBException
     */
    function downloadMest($output)
    {
        $mesta = [];

        $sql =
            'SELECT Mesto.PSC, Mesto.Mesto, Mesto.Posta, Mesto.Obec, Mesto.Stat, Mesto.Okres, Obec.Popis, Okres.Popis as PopisOkresu ' .
            ' FROM Mesto LEFT OUTER JOIN Obec ON (Mesto.Obec = Obec.Kod)' .
            ' LEFT OUTER JOIN Okres ON Obec.Okres = Okres.Kod' .
            ' ORDER BY Mesto.Mesto';

        $cursor = $this ->database->getSqlResult($sql);
        while ($row = $cursor->fetch()) {
            $mesta[] = [
                'psc' => StringUtils::jsonText($row[0]),
                'mesto' => StringUtils::jsonText($row[1]),
                'posta' => StringUtils::jsonText($row[2]),
                'obec' => StringUtils::jsonText($row[3]),
                'stat' => StringUtils::jsonText($row[4]),
                'okres' => StringUtils::jsonText($row[5]),
                'obecPopis' => StringUtils::jsonText($row[6]),
                'okresPopis' => StringUtils::jsonText($row[7])
            ];
        }
        $cursor ->close();

        $output ->addData('items', $mesta);
        $output->compress = 'gzip';
    }

    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function handleJsonAction($action, $output)
    {
        if (isset($_GET['seznamobci'])) {
            $this ->downloadObce($output);
            return;
        }

        if (isset($_GET['seznammest'])) {
            $this ->downloadMest($output);
            return;
        }

        throw new \Exception('Neznama akce!');
    }


    public function handleAction($action)
    {
        try {
            if (isset($_GET['css'])) {
                return $this ->downloadUserCss();
            }

            if (isset($_GET['login'])) {
                return $this->downloadDataOsoby();
            }

            if (isset($_GET['data'])) {
                switch ($_GET['data']) {
                    case 'absence' :
                        return $this->downloadAbsenceHromadne();
                }
            }

            if (isset($_GET['usersettings'])  &&  $_GET['usersettings']) {
                $text = NastaveniUzivatele::loadUserSettingsText($this ->database);
                return new DataOutputObject($text, 'nastaveni_uzivatele.txt', TextManager::getInstance() ->getCharSet());
            }

        } catch (\Exception $e) {
            $this ->renderError($e ->getMessage());
            return null;
        }

        return parent::handleAction($action);
    }

}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2206_162.zip: ronportal/ronportal/controller/downloadcontroller.php <==
<?php
/**
 * stanislav.elbl, 11.03.2021
 * Download dat (kalendare ical atd.)
 */

namespace RonPortal\Controller;

use Ron\CalendarItem;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\Http\HttpHeaders;
use Ron\MVC\DataOutputObject;
use Ron\MVC\IcalOutputObject;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dochazka\AbsenceCalendarItem;
use RonPortal\NastaveniUzivatele;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use RonPortal\SluzebniCesty\SluzebniCestaCalItem;

class DownloadController extends JsonController
{
    /** @var  Database */
    protected $database;

    /**
     * DownloadController constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        $this->database = $database;
        $this ->enableAccessWithoutLogin = true;
    }


    /**
     * @param $loginName
     * @param $jenSchvalene
     * @return AbsenceCalendarItem[]
     * @throws DBException
     */
    function absenceIcalEvents($loginName, $jenSchvalene)
    {
        $events = AbsenceCalendarItem::loadUserEvents($this ->database, $loginName, $jenSchvalene);
        return $events;
    }


    /**
     * @param $loginName
     * @return bool|SluzebniCestaCalItem|SluzebniCestaCalItem[]
     * @throws DBException
     */
    private function sluzebniCestyIcalEvents($loginName)
    {
        TextManager::getInstance() ->loadStrings('sluzebnicesty.ini');
        $items = SluzebniCestaCalItem::loadSluzebniCesty($this ->database, null, $loginName);
        return $items;
    }


    /**
     * @param $login
     * @return array
     * @throws DBException
     */
    function smenyIcalEvents($login)
    {
        $events = [];

        $params = $this ->database ->createParameters();

        $sql =
            'SELECT PlanovaniKapacit.Datum, PlanovaniKapacit.Smena, Smena.Popis, Smena.Variabilni, ' . // 0..3
            '       PlanovaniKapacit.Smena_Pocatek, PlanovaniKapacit.Smena_Konec, PlanovaniKapacit.Smena_Delka,' . // 4..6
            '       Smena.PocatekSmeny, Smena.KonecSmeny, Smena.DelkaSmeny, ' . // 7..9
            '       PracovniPomer.Id ' . // 10
            '  FROM PlanovaniKapacit, PracovniPomer, Smena, Osoba ' .
            ' WHERE PracovniPomer.RC = PlanovaniKapacit.RC ' .
            '   AND PracovniPomer.OsobniCislo = PlanovaniKapacit.OsobniCislo ' .
            '   AND PracovniPomer.DatumNastupu = PlanovaniKapacit.DatumNastupu ' .
            '   AND Smena.Kod = PlanovaniKapacit.Smena ' .
            '   AND Osoba.RC = PracovniPomer.RC ' .
            '   AND Osoba.IntrawebLogin = ' . $params->addParameter($login) .
            '   AND PlanovaniKapacit.Datum >= ' . $params ->addParameter($this ->database ->dateToParam(time())) .
            ' ORDER BY PlanovaniKapacit.Datum';

        $cursor = $this ->database ->getSqlResult($sql, $params);

        while ($row = $cursor ->fetch()) {
            $cal = new CalendarItem();

            $date = $this ->database ->stringToTimestamp($row[0]);
            if ($row[3]) {
                // variabilni
                $casOd = DateUtils::timeOf($this ->database ->stringToTimestamp($row[4]));
                $casDo = DateUtils::timeOf($this ->database ->stringToTimestamp($row[5]));
            } else {
                // pevna
                $casOd = DateUtils::timeOf($this ->database ->stringToTimestamp($row[7]));
                $casDo = DateUtils::timeOf($this ->database ->stringToTimestamp($row[8]));
            }

            // OSETRIT KONEC SMENY V DALSIM DNI!

            $cal->dateTimeFrom = $date + $casOd;
            $cal->dateTimeTo = $date + $casDo;

            $cal->uid = "ronportal.pk.{$row[10]}.$date";

            $cal->summary = StringUtils::fromUTF8('Směna ') . $row[1];

            $events[] = $cal;
        }

        $cursor ->close();

        return $events;
    }


    function renderError($text, $httpStatus = 'HTTP/1.0 404 Not Found')
    {
        header($httpStatus);
        header('Content-Type: text/plain; charset=windows-1250');
        echo $text;
        exit;
    }


    /**
     * @return IcalOutputObject
     * @throws \Exception
     * @throws DBException
     */
    function downloadDataOsoby()
    {
        $usersettings = new NastaveniUzivatele();
        if (!$usersettings->load($this ->database, null, $_GET['login'])) {
            // nepovedlo se nacist nastaveni - pravdepodobne neexistuje osoba s takovym loginem
            $this->renderError('Spatne prihlasovaci jmeno / bad login name!');
        }

        if (!$usersettings->getItem(NastaveniUzivatele::ICAL_KEY)) {
            // uzivatel nema vygenerovany platny klic
            $this ->renderError('Uzivatel nema vygenerovany klic!');
        }

        // $_GET['data'] - co se ma stahnout
        $filename = isset($_GET['filename']) ? $_GET['filename'] : 'calendar.ics'; // nepovinne jmeno souboru (pro testovani mozno zadat &filename='kalendar.txt')

        if ($_GET['key'] != $usersettings->getItem(NastaveniUzivatele::ICAL_KEY)) {
            // uzivatel nema vygenerovany platny klic
            $this ->renderError('Spatne zadany klic!', 'HTTP/1.0 401 Unauthorized');
        }

        $items = [];

        switch ($_GET['data']) {
            case 'absence' :
                $items = $this ->absenceIcalEvents($_GET['login'], true); // jen schvalene
                break;

            case 'absence1' :
                $items = $this ->absenceIcalEvents($_GET['login'], false); // jen schvalene = false
                break;

            case 'smeny' :
                $items = $this ->smenyIcalEvents($_GET['login']);
                break;

            case 'sluzebnicesty' :
                $items = $this ->sluzebniCestyIcalEvents($_GET['login']);
                break;

            case 'all' :
                // absence
                $itemsAbsence = null;
                if ($usersettings ->getItem(NastaveniUzivatele::ICAL_VSECHNY_ABSENCE)) {
                    $itemsAbsence = $this ->absenceIcalEvents($_GET['login'], false);
                } else if ($usersettings->getItem(NastaveniUzivatele::ICAL_SCHVALENE_ABSENCE)) {
                    $itemsAbsence = $this ->absenceIcalEvents($_GET['login'], true);
                }

                // smeny
                $itemsSmeny = $usersettings->getItem(NastaveniUzivatele::ICAL_SMENY)
                    ? $this ->smenyIcalEvents($_GET['login'])
                    : null;
                // sluzebni cesty
                $itemsSC = $usersettings->getItem(NastaveniUzivatele::ICAL_SLUZEBNI_CESTY)
                    ? $this ->sluzebniCestyIcalEvents($_GET['login'])
                    : null;

                if ($itemsAbsence) {
                    array_splice($items, 0, 0, $itemsAbsence);
                }
                if ($itemsSmeny) {
                    array_splice($items, 0, 0, $itemsSmeny);
                }
                if ($itemsSC) {
                    array_splice($items, 0, 0, $itemsSC);
                }
                break;

            default :
                throw new \Exception('Neznama akce!');
        }

        return new IcalOutputObject($items, $filename);
    }


    /**
     * @return IcalOutputObject
     * @throws DBException
     */
    function downloadAbsenceHromadne()
    {
        // seznam vsech absenci
        if (!RonPortalSettings::getIntValue('absence_hromadny_ical')) {
            $this ->renderError('Tato funkce neni povolena!');
        }

        if ($_GET['key'] != RonPortalSettings::getValue('absence_ical_key')) {
            $this ->renderError('Spatne zadany klic!', 'HTTP/1.0 401 Unauthorized');
        }

        $events = AbsenceCalendarItem::loadAllEvents($this ->database, true, (int)$_GET['typrozdeleni'], $_GET['rozdeleni']);

        $filename = isset($_GET['filename']) ? $_GET['filename'] : 'calendar.ics'; // nepovinne jmeno souboru (pro testovani mozno zadat &filename='kalendar.txt')
        return new IcalOutputObject($events, $filename);
    }


    public static function changeColor($htmlColor, $step = 10)
    {
        if (!$htmlColor  ||  $htmlColor[0] != '#') {
            return $htmlColor;
        }

        if (strlen($htmlColor) == 4) {
            $r = $htmlColor[1] . $htmlColor[1];
            $g = $htmlColor[2] . $htmlColor[2];
            $b = $htmlColor[3] . $htmlColor[3];
        } else if (strlen($htmlColor) == 7) {
            $r = $htmlColor[1] . $htmlColor[2];
            $g = $htmlColor[3] . $htmlColor[4];
            $b = $htmlColor[5] . $htmlColor[6];
        } else {
            return $htmlColor;
        }

        $r = max(0, min(255, hexdec($r) + $step));
        $g = max(0, min(255, hexdec($g) + $step));
        $b = max(0, min(255, hexdec($b) + $step));

        return
            '#' .
            str_pad(dechex($r), 2, '0', STR_PAD_LEFT) .
            str_pad(dechex($g), 2, '0', STR_PAD_LEFT) .
            str_pad(dechex($b), 2, '0', STR_PAD_LEFT);
    }


    function getBasicUserCss()
    {
        if (!RonPortalSettings::getIntValue('css_pouzit_barvy')) {
            return '';
        }

        $result = '';

        // hlavicka
        if ($color = RonPortalSettings::getValue('css_header_background')) {
            $result .=
                ".header { background: $color; }\n" .
                ".header-background { background: $color !important; }\n" .
                ".floatingmenu { border-color: $color; }\n";
        }
        if ($color = RonPortalSettings::getValue('css_header_color')) {
            $result .=
                ".header { color: $color; }\n" .
                ".header-color { color: $color !important; }\n";
        }
        if ($height = RonPortalSettings::getValue('css_header_height')) {
            $result .=
                ".header { height: $height; }\n" .
                ".floatingmenu.expanded .topmenu { padding-top: $height; }\n";
        }

        // nadpisy
        if ($color = RonPortalSettings::getValue('css_text_header_color')) {
            $result .=
                "h1, h2, .appcontent-headerdiv { color: $color; }\n";
        }

        // tlacitka
        if ($color = RonPortalSettings::getValue('css_button_background')) {
            $result .=
                ".submitbutton, .button { background: $color; }\n" .
                ".button-color { background: $color !important; }\n" .
                ".submitbutton, .button { border-color: " . self::changeColor($color, -20) . "; }\n" .
                ".button:hover, .submitbutton:hover { background: " . self::changeColor($color) . "; }\n";
        }
        if ($color = RonPortalSettings::getValue('css_button_color')) {
            $result .=
                ".submitbutton, .button { color: $color; }\n" .
                ".button-color { color: $color !important; }\n";
        }

        // zahlavi tabulek
        if ($color = RonPortalSettings::getValue('css_table_header_background')) {
            $result .=
                ".browser th, .floatingheader, .floatingheadercol { background: $color; }\n" .
                ".background-table-header { background: $color !important; }\n";
        }
        if ($color = RonPortalSettings::getValue('css_table_header_color')) {
            $result .=
                ".browser th, .browser th a, .floatingheader, .floatingheader a, .floatingheadercol, .floatingheadercol a { color: $color; }\n" .
                ".color-table-header { color: $color !important; }\n";
        }


        return $result;
    }

    /**
     * @return DataOutputObject
     * @throws \Exception
     */
    function downloadUserCss()
    {
        $text = $this->getBasicUserCss();

        $cursor = $this->database->select("SELECT Val FROM SYS_BVAR_GLOBAL WHERE Name = 'RonPortal_CSS'");
        $row = $cursor->fetch();
        $text .= $row ? $cursor ->stringToBinary($row[0]) : '';
        $cursor ->close();

        //HttpResponse::setHeader(HttpHeaders::LAST_MODIFIED, date('r', RonPortalSession::sessionTimestamp()));
        //HttpResponse::setHeader(HttpHeaders::EXPIRES, date('r', DateUtils::incDay(RonPortalSession::sessionTimestamp(), 1)));
        $result = new DataOutputObject($text, 'user_styles.css');
        $result ->headers[HttpHeaders::CACHE_CONTROL] = 'private, max-age=3600';
        //$result ->headers[HttpHeaders::CACHE_CONTROL] = 'private';
        $result ->headers[HttpHeaders::PRAGMA] = 'private';
        $result ->headers[HttpHeaders::EXPIRES] = date('D, d M Y H:i:s', DateUtils::incDay(RonPortalSession::sessionTimestamp(), 1)) . ' GMT';
        return $result;
    }

    /**
     * @param JsonOutputObject $output
     * @param bool $emptyValue
     * @throws DBException
     */
    function downloadObce($output, $emptyValue = false)
    {
        $obce = [];

        if ($emptyValue) {
            $obce[] = ['kod' => '', 'popis' => ''];
        }

        $sql =
            'SELECT Obec.Kod, Obec.Popis, Obec.Okres, Okres.Popis' .
            ' FROM Obec JOIN Okres ON Obec.Okres = Okres.Kod' .
            ' ORDER BY Obec.Popis';

        $cursor = $this ->database->getSqlResult($sql);
        while ($row = $cursor->fetch()) {
            $obce[] = [
                'kod' => StringUtils::jsonText($row[0]),
                'popis' => StringUtils::jsonText($row[1]),
                'okres' => StringUtils::jsonText($row[2]),
                'okresPopis' => StringUtils::jsonText($row[3])
            ];
        }
        $cursor ->close();

        $output ->addData('items', $obce);
        $output->compress = 'gzip';
    }

    /**
     * @param JsonOutputObject $output
     * @throws DBException
     */
    function downloadMest($output)
    {
        $mesta = [];

        $sql =
            'SELECT Mesto.PSC, Mesto.Mesto, Mesto.Posta, Mesto.Obec, Mesto.Stat, Mesto.Okres, Obec.Popis, Okres.Popis as PopisOkresu ' .
            ' FROM Mesto LEFT OUTER JOIN Obec ON (Mesto.Obec = Obec.Kod)' .
            ' LEFT OUTER JOIN Okres ON Obec.Okres = Okres.Kod' .
            ' ORDER BY Mesto.Mesto';

        $cursor = $this ->database->getSqlResult($sql);
        while ($row = $cursor->fetch()) {
            $mesta[] = [
                'psc' => StringUtils::jsonText($row[0]),
                'mesto' => StringUtils::jsonText($row[1]),
                'posta' => StringUtils::jsonText($row[2]),
                'obec' => StringUtils::jsonText($row[3]),
                'stat' => StringUtils::jsonText($row[4]),
                'okres' => StringUtils::jsonText($row[5]),
                'obecPopis' => StringUtils::jsonText($row[6]),
                'okresPopis' => StringUtils::jsonText($row[7])
            ];
        }
        $cursor ->close();

        $output ->addData('items', $mesta);
        $output->compress = 'gzip';
    }

    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws \Exception
     */
    function handleJsonAction($action, $output)
    {
        if (isset($_GET['seznamobci'])) {
            $this ->downloadObce($output, isset($_GET['showemptyvalue']) && (int)$_GET['showemptyvalue']);
            return;
        }

        if (isset($_GET['seznammest'])) {
            $this ->downloadMest($output);
            return;
        }

        throw new \Exception('Neznama akce!');
    }


    public function handleAction($action)
    {
        try {
            if (isset($_GET['css'])) {
                return $this ->downloadUserCss();
            }

            if (isset($_GET['login'])) {
                return $this->downloadDataOsoby();
            }

            if (isset($_GET['data'])) {
                switch ($_GET['data']) {
                    case 'absence' :
                        return $this->downloadAbsenceHromadne();
                }
            }

            if (isset($_GET['usersettings'])  &&  $_GET['usersettings']) {
                $text = NastaveniUzivatele::loadUserSettingsText($this ->database);
                return new DataOutputObject($text, 'nastaveni_uzivatele.txt', TextManager::getInstance() ->getCharSet());
            }

        } catch (\Exception $e) {
            $this ->renderError($e ->getMessage());
            return null;
        }

        return parent::handleAction($action);
    }

}
